import Flutter
import UIKit
import Smile_Identity_SDK

#if arch(arm64) || arch(x86_64)
public class SwiftSmileFlutterPlugin: NSObject, FlutterPlugin,SIDCaptureManagerDelegate {
    
    
    let SID_RESULT_CODE = "SID_RESULT_CODE"
    let SID_RESULT_MESSAGE = "SID_RESULT_MESSAGE"
    let SID_RESULT_TAG = "SID_RESULT_TAG"
    var currentResult : FlutterResult?
    var currentTag:String?
    let mainIdInfo = ["first_name", "last_name", "middle_name", "country", "id_type", "id_number", "email"]
    let mainPartnerParams = ["user_id", "job_id"]
    
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "smile_flutter", binaryMessenger: registrar.messenger())
        let instance = SwiftSmileFlutterPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        var contextTag = ""
        if let args = call.arguments as? Dictionary<String, Any>{
            contextTag = args["tag"] as? String ?? ""
        }
        
        switch call.method {
        case "getPlatformVersion":
            result("iOS " + UIDevice.current.systemVersion)
            return
        case "captureSelfie":
            smileCapture(tag: contextTag, captureType: CaptureType.SELFIE, result: result)
            return
        case "captureIDCard":
            smileCapture(tag: contextTag, captureType: CaptureType.ID_CAPTURE, result: result)
            return
        case "captureSelfieAndIDCard":
            smileCapture(tag: contextTag, captureType: CaptureType.SELFIE_AND_ID_CAPTURE, result: result)
            return
        case "submitJob":
            var jobType : Int?
            var isProduction : Bool?
            var callBackUrl : String?
            var partnerParams :  Dictionary<String, String>?
            var idInfo : Dictionary<String, String>?
            var geoInfo : Dictionary<String, String>?
            if let args = call.arguments as? Dictionary<String, Any>{
                jobType = args["jobType"] as? Int ?? -1
                isProduction = args["isProduction"] as? Bool ?? false
                callBackUrl = args["callBackUrl"] as? String ?? ""
                partnerParams = args["partnerParams"] as? Dictionary<String, String> ?? nil
                idInfo = args["idInfo"] as? Dictionary<String, String> ?? nil
                geoInfo = args["geoInfo"] as? Dictionary<String, String> ?? nil
            }
            submitJob(tag: contextTag, jobType: jobType , isProduction: isProduction ?? false,callBackUrl:callBackUrl, partnerParams: partnerParams, idInfo: idInfo, geoInfo: geoInfo, result:result)
            return
        default:
            result(FlutterError.init(code: "unimplemented", message: nil, details: nil))
        }
    }
    
    public func onSuccess(tag: String, selfiePreview: UIImage?, idFrontPreview: UIImage?, idBackPreview: UIImage?) {
        if let result = self.currentResult {
            let response = [self.SID_RESULT_TAG: tag, self.SID_RESULT_CODE: -1] as [String : Any]
            result(response)
        }
    }
    
    public func onError(tag: String, sidError: SIDError) {
        if let result = self.currentResult {
            let response = [self.SID_RESULT_TAG: tag, self.SID_RESULT_CODE: 0] as [String : Any]
            result(response)
        }
    }
    
    
    func getRetryOnFailurePolicy() -> RetryOnFailurePolicy {
        let options = RetryOnFailurePolicy();
        options.setMaxRetryTimeoutSec(maxRetryTimeoutSec:15 )
        return options;
    }
    
    func submitJob(tag: String, jobType: Int?, isProduction: Bool?,callBackUrl:String?, partnerParams: Dictionary<String, String>?,  idInfo: Dictionary<String, String>?,  geoInfo: Dictionary<String, Any>?, result: @escaping FlutterResult) {
        let sidNetworkRequest = SIDNetworkRequest()
        sidNetworkRequest.setDelegate(delegate: SubmitJobListener(result: result))
        sidNetworkRequest.initialize()
        let sidNetData = SIDNetData(environment: isProduction ?? false ? SIDNetData.Environment.PROD : SIDNetData.Environment.TEST);
        if let callBack = callBackUrl{
            sidNetData.setCallBackUrl(callbackUrl: callBack)
        }
        let sidConfig = SIDConfig()
        sidConfig.setSidNetworkRequest( sidNetworkRequest : sidNetworkRequest )
        sidConfig.setSidNetData( sidNetData : sidNetData )
        sidConfig.setRetryOnFailurePolicy( retryOnFailurePolicy: getRetryOnFailurePolicy() )
        if let idInfo = idInfo{
            let sidIdInfo = SIDUserIdInfo()
            for (key, value) in idInfo {
                switch key {
                case "country":
                    sidIdInfo.setCountry(country: value )
                case "id_type":
                    sidIdInfo.setIdType(idType: value )
                case "id_number":
                    sidIdInfo.setIdNumber(idNumber: value )
                case "first_name":
                    sidIdInfo.setFirstName(firstName: value )
                case "middle_name":
                    sidIdInfo.setMiddleName(middleName: value )
                case "last_name":
                    sidIdInfo.setLastName(lastName: value )
                case "email":
                    sidIdInfo.setEmail(email: value )
                default:
                    sidIdInfo.additionalValue(name: key , value: value as! String)
                }
            }
            sidConfig.setUserIdInfo(userIdInfo: sidIdInfo)
        }
    
        let sidPartnerParams = PartnerParams()
        if let partnerParams = partnerParams{
            for (key, value) in partnerParams {
                switch key {
                case "job_id":
                    sidPartnerParams.setJobId(jobId: value )
                case "user_id":
                    sidPartnerParams.setUserId(userId:  value )
                default:
                    sidPartnerParams.setAdditionalValue(key: key , val: value as! String)
                }
            }
        }
        sidPartnerParams.setJobType(jobType: jobType! )
        sidConfig.setPartnerParams( partnerParams : sidPartnerParams )
        
        if let geoInfo = geoInfo {
            
            let sidGeoInfo = GeoInfos(latitude: geoInfo["latitude"] as! Double, longitude: geoInfo["longitude"] as! Double, altitude: geoInfo["altitude"] as! Double, accuracy: geoInfo[ "accuracy"] as! Double, lastUpdate: geoInfo["lastUpdate"] as! String)
        }
        let isEnrollMode = jobType == 1 || jobType == 4
        sidConfig.setIsEnrollMode(isEnrollMode: jobType == 1 || jobType == 4)
        if(isEnrollMode){
            let hasIdCard = SIDInfosManager.hasIdCard(userTag: tag)
            sidConfig.setUseIdCard(useIdCard: hasIdCard)
        }
        sidConfig.build(userTag: tag)
        do {
            try sidConfig.getSidNetworkRequest().submit(sidConfig: sidConfig)
        } catch {
            result(FlutterError.init(code: "Oops something went wrong!", message: nil, details: nil))
        }
    }
    
    func smileCapture(tag: String, captureType: CaptureType, result: @escaping FlutterResult){
        assignTempVars(tag: tag, result:result)
        DispatchQueue.main.async {
            let builder = SIDCaptureManager.Builder(delegate:self, captureType: captureType)
            if !tag.isEmpty {
                builder.setTag(tag: tag)
            }
            if(captureType == CaptureType.SELFIE_AND_ID_CAPTURE || captureType == CaptureType.ID_CAPTURE){
                let sidIdCaptureConfig = SIDIDCaptureConfig.Builder().setIdCaptureType(idCaptureType: IDCaptureType.Front).build()
                builder.setSidIdCaptureConfig(sidIdCaptureConfig: sidIdCaptureConfig!)
            }
            builder.build().start()
        }
    }
    
    func assignTempVars(tag: String,result: @escaping FlutterResult){
        currentResult = result
        self.currentTag = tag
    }
    
    func deassignTempVar(){
        currentResult = nil
        currentTag = nil
    }
}

class SubmitJobListener: SIDNetworkRequestDelegate{
    let result:FlutterResult
    init(result:@escaping FlutterResult) {
        self.result = result
    }
    func onStartJobStatus() {
        
    }
    
    func onEndJobStatus() {
        
    }
    
    func onUpdateJobProgress(progress: Int) {
        //            let params = ["status": "\(progress)"]
        //            SIDEventEmitter.sharedInstance.dispatch(name: "UploadListener", body: params)
    }
    
    func onUpdateJobStatus(msg: String) {
        
    }
    
    func onAuthenticated(sidResponse: SIDResponse) {
        if let response = sidResponse.getStatusResponse(){
            let response = convertToDictionary(text: response.getRawJsonString())
            self.result(response)
            return
        }
        result(FlutterError.init(code: "onAuthenticated Failed", message: nil, details: nil))
    }
    
    func onEnrolled(sidResponse: SIDResponse) {
        if let response = sidResponse.getStatusResponse(){
            let toReturn = convertToDictionary(text: response.getRawJsonString())
            if(toReturn != nil){
                self.result(toReturn)
                return
            }

        }
        result(FlutterError.init(code: "onEnrolled Failed", message: nil, details: nil))
    }
    
    func onComplete() {
        //        let params = ["status": "done"]
        //        SIDEventEmitter.sharedInstance.dispatch(name: "CompleteListener", body: params)
        
    }
    
    func onError(sidError: SIDError) {
        result(FlutterError.init(code:  "onError", message: sidError.message, details: nil))
    }
    
    func onIdValidated(idValidationResponse: IDValidationResponse) {
        let response = convertToDictionary(text: idValidationResponse.getRawJsonString())
        if(response != nil){
            self.result(response)
            return
        }
        result(FlutterError.init(code: "onIdValidated Failed", message: nil, details: nil))
    }
    
    func convertToDictionary(text: String) -> [String: Any]? {
        if let data = text.data(using: .utf8) {
            do {
                return try JSONSerialization.jsonObject(with: data, options: []) as? [String: Any]
            } catch {
                result(FlutterError.init(code: "Oops something went wrong", message: nil, details: nil))
            }
        }
        return nil
    }
}
#else
public class SwiftSmileFlutterPlugin: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
       
    }
}
#endif
